@@ -8,14 +8,14 @@ import sh
 class SDL2GradleBootstrap(Bootstrap):
     name = 'sdl2'
 
-    recipe_depends = ['sdl2']
+    recipe_depends = list(
+        set(Bootstrap.recipe_depends).union({'sdl2'})
+    )
 
     def run_distribute(self):
         info_main("# Creating Android project ({})".format(self.name))
 
         arch = self.ctx.archs[0]
-        python_install_dir = self.ctx.get_python_install_dir()
-        from_crystax = self.ctx.python_recipe.from_crystax
 
         if len(self.ctx.archs) > 1:
             raise ValueError("SDL2/gradle support only one arch")
@@ -34,11 +34,6 @@ class SDL2GradleBootstrap(Bootstrap):
             info("Copying Python distribution")
 
             python_bundle_dir = join('_python_bundle', '_python_bundle')
-            if 'python2legacy' in self.ctx.recipe_build_order:
-                # a special case with its own packaging location
-                python_bundle_dir = 'private'
-                # And also must had an install directory, make sure of that
-                self.ctx.python_recipe.create_python_install(self.dist_dir)
 
             self.distribute_libs(arch, [self.ctx.get_libs_dir(arch.arch)])
             self.distribute_javaclasses(self.ctx.javaclass_dir,
@@ -52,9 +47,10 @@ class SDL2GradleBootstrap(Bootstrap):
                 with open('blacklist.txt', 'a') as fileh:
                     fileh.write('\nsqlite3/*\nlib-dynload/_sqlite3.so\n')
 
-        self.strip_libraries(arch)
+        if not self.ctx.build_as_debuggable:
+            self.strip_libraries(arch)
         self.fry_eggs(site_packages_dir)
-        super(SDL2GradleBootstrap, self).run_distribute()
+        super().run_distribute()
 
 
 bootstrap = SDL2GradleBootstrap()
