@@ -2,7 +2,7 @@ from pythonforandroid.recipe import PythonRecipe
 
 
 class Enum34Recipe(PythonRecipe):
-    version = '1.1.3'
+    version = '1.1.6'
     url = 'https://pypi.python.org/packages/source/e/enum34/enum34-{version}.tar.gz'
     depends = ['setuptools']
     site_packages_name = 'enum'
@@ -16,7 +16,7 @@ class Enum34Recipe(PythonRecipe):
             # attribute `depends` because otherwise we will not be able to
             # build the cryptography recipe.
             return False
-        return super(Enum34Recipe, self).should_build(arch)
+        return super().should_build(arch)
 
 
 recipe = Enum34Recipe()
