@@ -4,7 +4,7 @@ import sh
 
 
 class FFMpegRecipe(Recipe):
-    version = 'n3.4.5'
+    version = '007e03348dbd8d3de3eb09022d72c734a8608144'
     # Moved to github.com instead of ffmpeg.org to improve download speed
     url = 'https://github.com/FFmpeg/FFmpeg/archive/{version}.zip'
     depends = ['sdl2']  # Need this to build correct recipe order
@@ -19,7 +19,7 @@ class FFMpegRecipe(Recipe):
         self.apply_patches(arch)
 
     def get_recipe_env(self, arch):
-        env = super(FFMpegRecipe, self).get_recipe_env(arch)
+        env = super().get_recipe_env(arch)
         env['NDK'] = self.ctx.ndk_dir
         return env
 
@@ -37,14 +37,17 @@ class FFMpegRecipe(Recipe):
                     '--enable-nonfree',
                     '--enable-protocol=https,tls_openssl',
                 ]
-                build_dir = Recipe.get_recipe('openssl', self.ctx).get_build_dir(arch.arch)
-                cflags += ['-I' + build_dir + '/include/']
+                build_dir = Recipe.get_recipe(
+                    'openssl', self.ctx).get_build_dir(arch.arch)
+                cflags += ['-I' + build_dir + '/include/',
+                           '-DOPENSSL_API_COMPAT=0x10002000L']
                 ldflags += ['-L' + build_dir]
 
             if 'ffpyplayer_codecs' in self.ctx.recipe_build_order:
                 # libx264
                 flags += ['--enable-libx264']
-                build_dir = Recipe.get_recipe('libx264', self.ctx).get_build_dir(arch.arch)
+                build_dir = Recipe.get_recipe(
+                    'libx264', self.ctx).get_build_dir(arch.arch)
                 cflags += ['-I' + build_dir + '/include/']
                 ldflags += ['-lx264', '-L' + build_dir + '/lib/']
 
@@ -53,6 +56,7 @@ class FFMpegRecipe(Recipe):
                 build_dir = Recipe.get_recipe('libshine', self.ctx).get_build_dir(arch.arch)
                 cflags += ['-I' + build_dir + '/include/']
                 ldflags += ['-lshine', '-L' + build_dir + '/lib/']
+                ldflags += ['-lm']
 
                 # Enable all codecs:
                 flags += [
@@ -79,39 +83,50 @@ class FFMpegRecipe(Recipe):
 
             # disable binaries / doc
             flags += [
-                '--disable-ffmpeg',
-                '--disable-ffplay',
-                '--disable-ffprobe',
-                '--disable-ffserver',
+                '--disable-programs',
                 '--disable-doc',
             ]
 
             # other flags:
             flags += [
                 '--enable-filter=aresample,resample,crop,adelay,volume,scale',
-                '--enable-protocol=file,http',
+                '--enable-protocol=file,http,hls',
                 '--enable-small',
                 '--enable-hwaccels',
                 '--enable-gpl',
                 '--enable-pic',
                 '--disable-static',
+                '--disable-debug',
                 '--enable-shared',
             ]
 
+            if 'arm64' in arch.arch:
+                cross_prefix = 'aarch64-linux-android-'
+                arch_flag = 'aarch64'
+            else:
+                cross_prefix = 'arm-linux-androideabi-'
+                arch_flag = 'arm'
+
             # android:
             flags += [
                 '--target-os=android',
-                '--cross-prefix=arm-linux-androideabi-',
-                '--arch=arm',
-                '--sysroot=' + self.ctx.ndk_platform,
+                '--enable-cross-compile',
+                '--cross-prefix={}-'.format(arch.target),
+                '--arch={}'.format(arch_flag),
+                '--strip={}strip'.format(cross_prefix),
+                '--sysroot={}'.format(join(self.ctx.ndk_dir, 'toolchains',
+                                           'llvm', 'prebuilt', 'linux-x86_64',
+                                           'sysroot')),
                 '--enable-neon',
                 '--prefix={}'.format(realpath('.')),
             ]
-            cflags += [
-                '-mfpu=vfpv3-d16',
-                '-mfloat-abi=softfp',
-                '-fPIC',
-            ]
+
+            if arch_flag == 'arm':
+                cflags += [
+                    '-mfpu=vfpv3-d16',
+                    '-mfloat-abi=softfp',
+                    '-fPIC',
+                ]
 
             env['CFLAGS'] += ' ' + ' '.join(cflags)
             env['LDFLAGS'] += ' ' + ' '.join(ldflags)
@@ -121,7 +136,8 @@ class FFMpegRecipe(Recipe):
             shprint(sh.make, '-j4', _env=env)
             shprint(sh.make, 'install', _env=env)
             # copy libs:
-            sh.cp('-a', sh.glob('./lib/lib*.so'), self.ctx.get_libs_dir(arch.arch))
+            sh.cp('-a', sh.glob('./lib/lib*.so'),
+                  self.ctx.get_libs_dir(arch.arch))
 
 
 recipe = FFMpegRecipe()
